# Theory: Conflict resolution

* A conflict emerges when two files push two different changes
to a one target branch

* Some conflict may be resolved automatically, but major conflicts
always need to be resolved manually

* Tools exist to streamline conflict resolutions, we use kdiff3



# Practical: Conflict resolution using kdiff3

* create a branch named changeMicheal and in the editor you change
the content of `Michael.md`:
```bash
$ git checkout develop
$ git checkout -b changeMichael
```

Change Michael file...

```bash
$ git fetch upstream
$ git checkout develop
$ git merge upstream/develop
```

summary:
michael.md on develop:
```
# Michael
```

michael.md on changeMichael (example)
```
## Michael Evans
```



# Practical: Conflict resolution using kdiff3

If then there is a rebase done on develop:
```bash
$ git rebase develop
```
A conflict should emerge

* When facing the merge conflict message, use the command
```bash
$ git mergetool
```

* This opens kdiff3 if it was properly set up. Selecting A will keep the changes of the target branch,
while B will keep your changes. Selecting A and B will combine both changes to the merged file.




# Practical: Conflict resolution using kdiff3

* After resolving the conflicts, continue in your terminal
```bash
$ git rebase --continue
```
* Then check the status:
```bash
$ git status
```

* It shows you a `.orig` file. This `.orig` file contains information about the conflict.
```bash
$ rm _attendees/michael.md.orig
```
* You can either `rm` the `.orig` file, or you can use `git clean -fdx`.