# Theory: Git Rebasing

* Git rebase enables to keep up with changes made to a branch
* Straightens the workflow
* Avoid discrepancies when multiple people work on the same project

Imagine the following situation:
<div style="top: 10em; left: 30%; position: absolute;">
    <img src="slides/img/beforeRebase.png" height="400px">
</div>



# Theory: Git Rebasing

* Develop branch is several commits ahead of master.
* Commits implemented into master that aren't in develop.

<div style="top: 10em; left: 30%; position: absolute;">
    <img src="slides/img/afterRebase.png" height="400px">
</div>



# Practical: Git Rebasing

* Create a file in your branch
```bash
$ git checkout myBranch
$ echo "# message" > yourName.md
```

* Then create a file in the develop branch
```bash
$ git checkout develop
$ touch startRebase.txt
```

* Check the histories of both branches
```bash
$ git log
```



# Practical: Git Rebasing

* rebase the develop branch onto your branch
```bash
$ git checkout myBranch
$ git rebase develop
```

* check the history of your branch again
```bash
$ git log
```



# Theory: Git Interactive Rebasing

```bash
git rebase -i <branch>
```

* Enables more precise control over the rebased commits
* Before committing many actions are at disposal

```bash
#  p, pick = use commit
#  r, reword = use commit, but edit the commit message
#  e, edit = use commit, but stop for amending
#  s, squash = use commit, but meld into previous commit
#  f, fixup = like "squash", but discard this commit's log message
#  x, exec = run command (the rest of the line) using shell
```



# Practical: Git Interactive Rebasing

* Create and switch to your own branch
* Add and commit two files to this branch:

```bash
$ echo "# Michael" > Michael.md
$ git add Michael.md
$ git commit -m "Add Michael to the list of attendees"
$ git push origin yourBranch

$ echo "# Kevin" > Kevin.md
$ git add Kevin.md
$ git commit -m "Add Kevin to the list of attendees"
$ git push origin yourBranch
```



# Practical: Git Interactive Rebasing

* Perform an interactive rebase with the two last commits:

```bash
$ git rebase -i HEAD~2
```

* reword the first commit's message to:
`These are the two squashed commits`
* combine the second and first commit into one,
omitting the second's commit message.



# Practical: Git Interactive Rebasing

The prompt shows up:
```bash
$ pick 1234567 add Micheal to attendee list
$ pick abcdef0 add Kevin to attendee list
```

The keywords should be changed to:
```bash
$ reword 1234567 add Micheal to attendee list
$ fixup abcdef0 add Kevin to attendee list
```

Upon confirming, change the message of commit 1234567:
```bash
$ Add Micheal and Kevin to the attendee list
```

Then use force-push to push your changes:
```bash
$ git push origin myBranch -f
```



# Practical: Git Interactive Rebasing

* If you wanted to keep both commit messages in one commit, instead change the prompts to squashing instead
```bash
$ pick 1234567 add Micheal to attendee list
$ squash abcdef0 add Kevin to attendee list
```

* This will create a commit with both files still in, but the commit message of that commit will be the two commit messages of the two files combined.